@@ -1,8 +1,7 @@
-# Copyright (c) 2019 Microsoft Corporation
+# Copyright (c) 2023 The InterpretML Contributors
 # Distributed under the MIT software license
 
 from ..api.base import ExplainerMixin, ExplanationMixin
-from ..utils import unify_data
 from ..utils import (
     gen_name_from_class,
     gen_local_selector,
@@ -14,11 +13,21 @@ from sklearn.base import ClassifierMixin, RegressorMixin
 from sklearn.tree import DecisionTreeClassifier as SKDT
 from sklearn.tree import DecisionTreeRegressor as SKRT
 from sklearn.base import is_classifier
+from sklearn.utils.validation import check_is_fitted
 import numpy as np
 from abc import abstractmethod
 from sklearn.tree import _tree
 from copy import deepcopy
 
+
+from ..utils._binning import (
+    preclean_X,
+    unify_data,
+    clean_dimensions,
+    typify_classification,
+)
+
+
 import logging
 
 log = logging.getLogger(__name__)
@@ -27,7 +36,7 @@ COLORS = ["#1f77b4", "#ff7f0e", "#808080", "#3a729b", "#ff420e"]
 
 
 class TreeExplanation(ExplanationMixin):
-    """ Explanation object specific to trees. """
+    """Explanation object specific to trees."""
 
     explanation_type = None
 
@@ -40,7 +49,7 @@ class TreeExplanation(ExplanationMixin):
         name=None,
         selector=None,
     ):
-        """ Initializes class.
+        """Initializes class.
 
         Args:
             explanation_type:  Type of explanation.
@@ -59,7 +68,7 @@ class TreeExplanation(ExplanationMixin):
         self.selector = selector
 
     def data(self, key=None):
-        """ Provides specific explanation data.
+        """Provides specific explanation data.
 
         Args:
             key: A number/string that references a specific data item.
@@ -72,7 +81,7 @@ class TreeExplanation(ExplanationMixin):
         return self._internal_obj["specific"][key]
 
     def visualize(self, key=None):
-        """ Provides interactive visualizations.
+        """Provides interactive visualizations.
 
         Args:
             key: Either a scalar or list
@@ -222,9 +231,9 @@ class TreeExplanation(ExplanationMixin):
 
 
 class BaseShallowDecisionTree:
-    """ Shallow Decision Tree (low depth).
+    """Shallow Decision Tree (low depth).
 
-    Currently wrapper around DecisionTreeClassifier in scikit-learn.
+    Currently wrapper around DecisionTreeClassifier or DecisionTreeRegressor in scikit-learn.
     To keep the tree shallow, max depth is defaulted to 3.
 
     https://github.com/scikit-learn/scikit-learn
@@ -234,14 +243,14 @@ class BaseShallowDecisionTree:
     available_explanations = ["global", "local"]
     explainer_type = "model"
 
-    def __init__(self, max_depth=3, feature_names=None, feature_types=None, **kwargs):
-        """ Initializes tree with low depth.
+    def __init__(self, feature_names=None, feature_types=None, max_depth=3, **kwargs):
+        """Initializes tree with low depth.
 
         Args:
-            max_depth: Max depth of tree.
             feature_names: List of feature names.
             feature_types: List of feature types.
-            **kwargs: Kwargs sent to fit() method of tree.
+            max_depth: Max depth of tree.
+            **kwargs: Kwargs sent to __init__() method of tree.
         """
         self.feature_names = feature_names
         self.feature_types = feature_types
@@ -254,7 +263,7 @@ class BaseShallowDecisionTree:
         return None
 
     def fit(self, X, y):
-        """ Fits model to provided instances.
+        """Fits model to provided instances.
 
         Args:
             X: Numpy array for training instances.
@@ -263,21 +272,56 @@ class BaseShallowDecisionTree:
         Returns:
             Itself.
         """
-        X, y, self.feature_names, self.feature_types = unify_data(
-            X, y, self.feature_names, self.feature_types
-        )
-        sk_model_ = self._model()
-        sk_model_.fit(X, y)
 
-        feat_imp = sk_model_.feature_importances_
-        self.global_selector = gen_global_selector(
-            X, self.feature_names, self.feature_types, feat_imp
+        y = clean_dimensions(y, "y")
+        if y.ndim != 1:
+            raise ValueError("y must be 1 dimensional")
+        if len(y) == 0:
+            raise ValueError("y cannot have 0 samples")
+
+        if is_classifier(self):
+            y = typify_classification(y)
+        else:
+            y = y.astype(np.float64, copy=False)
+
+        X, n_samples = preclean_X(X, self.feature_names, self.feature_types, len(y))
+
+        X, self.feature_names_in_, self.feature_types_in_ = unify_data(
+            X, n_samples, self.feature_names, self.feature_types, False, 0
         )
-        self.n_samples_ = X.shape[0]
+
+        model = self._model()
+        model.fit(X, y)
+
+        unique_val_counts = np.zeros(len(self.feature_names_in_), dtype=np.int64)
+        zero_val_counts = np.zeros(len(self.feature_names_in_), dtype=np.int64)
+        for col_idx in range(len(self.feature_names_in_)):
+            X_col = X[:, col_idx]
+            unique_val_counts.itemset(col_idx, len(np.unique(X_col)))
+            zero_val_counts.itemset(col_idx, len(X_col) - np.count_nonzero(X_col))
+
+        feat_imp = model.feature_importances_
+        self.global_selector_ = gen_global_selector(
+            n_samples,
+            len(self.feature_names_in_),
+            self.feature_names_in_,
+            self.feature_types_in_,
+            unique_val_counts,
+            zero_val_counts,
+            feat_imp,
+        )
+        self.n_samples_ = n_samples
+
+        self.n_features_in_ = len(self.feature_names_in_)
+        if is_classifier(self):
+            self.classes_ = model.classes_
+
+        self.has_fitted_ = True
+
         return self
 
     def predict(self, X):
-        """ Predicts on provided instances.
+        """Predicts on provided instances.
 
         Args:
             X: Numpy array for instances.
@@ -285,11 +329,18 @@ class BaseShallowDecisionTree:
         Returns:
             Predicted class label per instance.
         """
-        X, _, _, _ = unify_data(X, None, self.feature_names, self.feature_types)
+
+        check_is_fitted(self, "has_fitted_")
+
+        X, n_samples = preclean_X(X, self.feature_names_in_, self.feature_types_in_)
+        X, _, _ = unify_data(
+            X, n_samples, self.feature_names_in_, self.feature_types_in_, False, 0
+        )
+
         return self._model().predict(X)
 
     def explain_global(self, name=None):
-        """ Provides global explanation for model.
+        """Provides global explanation for model.
 
         Args:
             name: User-defined explanation name.
@@ -298,27 +349,30 @@ class BaseShallowDecisionTree:
             An explanation object,
             visualizing feature-value pairs as horizontal bar chart.
         """
+
+        check_is_fitted(self, "has_fitted_")
+
         if name is None:
             name = gen_name_from_class(self)
 
         # Extract decision tree structure
         nodes, edges = self._graph_from_tree(
-            self._model(), self.feature_names, max_depth=self.max_depth
+            self._model(), self.feature_names_in_, max_depth=self.max_depth
         )
         overall_data_dict = {
             "type": "tree",
-            "features": self.feature_names,
+            "features": self.feature_names_in_,
             "nodes": nodes,
             "edges": edges,
         }
         data_dicts = [
             {
                 "type": "tree",
-                "features": self.feature_names,
+                "features": self.feature_names_in_,
                 "nodes": nodes,
                 "edges": edges,
             }
-            for _ in self.feature_names
+            for _ in self.feature_names_in_
         ]
 
         internal_obj = {"overall": overall_data_dict, "specific": data_dicts}
@@ -326,14 +380,14 @@ class BaseShallowDecisionTree:
         return TreeExplanation(
             "global",
             internal_obj,
-            feature_names=self.feature_names,
-            feature_types=self.feature_types,
+            feature_names=self.feature_names_in_,
+            feature_types=self.feature_types_in_,
             name=name,
-            selector=self.global_selector,
+            selector=self.global_selector_,
         )
 
     def explain_local(self, X, y=None, name=None):
-        """ Provides local explanations for provided instances.
+        """Provides local explanations for provided instances.
 
         Args:
             X: Numpy array for X to explain.
@@ -343,30 +397,59 @@ class BaseShallowDecisionTree:
         Returns:
             An explanation object.
         """
+
+        check_is_fitted(self, "has_fitted_")
+
         if name is None:
             name = gen_name_from_class(self)
 
-        X, y, _, _ = unify_data(X, y, self.feature_names, self.feature_types)
+        n_samples = None
+        if y is not None:
+            y = clean_dimensions(y, "y")
+            if y.ndim != 1:
+                raise ValueError("y must be 1 dimensional")
+            n_samples = len(y)
+
+            if is_classifier(self):
+                y = typify_classification(y)
+            else:
+                y = y.astype(np.float64, copy=False)
+
+        X, n_samples = preclean_X(
+            X, self.feature_names_in_, self.feature_types_in_, n_samples
+        )
+
+        if n_samples == 0:
+            # TODO: we could probably handle this case
+            raise ValueError("X has zero samples")
 
         # Extract decision tree structure
-        nodes, edges = self._graph_from_tree(self._model(), self.feature_names)
+        nodes, edges = self._graph_from_tree(self._model(), self.feature_names_in_)
+
+        X, _, _ = unify_data(
+            X, n_samples, self.feature_names_in_, self.feature_types_in_, False, 0
+        )
 
         decisions = [
             self._model().decision_path(instance.reshape(1, -1)).nonzero()[1] + 1
             for instance in X
         ]
 
+        classes = None
         is_classification = is_classifier(self)
         if is_classification:
-            predictions = self.predict_proba(X)[:, 1]
+            classes = self.classes_
+            predictions = self.predict_proba(X)
+            if len(self.classes_) == 2:
+                predictions = predictions[:, 1]
         else:
             predictions = self.predict(X)
 
-        perf_dicts = gen_perf_dicts(predictions, y, is_classification)
+        perf_dicts = gen_perf_dicts(predictions, y, is_classification, classes)
         data_dicts = [
             {
                 "type": "tree",
-                "features": self.feature_names,
+                "features": self.feature_names_in_,
                 "nodes": nodes,
                 "edges": edges,
                 "decision": decision,
@@ -381,15 +464,15 @@ class BaseShallowDecisionTree:
         return TreeExplanation(
             "local",
             internal_obj,
-            feature_names=self.feature_names,
-            feature_types=self.feature_types,
+            feature_names=self.feature_names_in_,
+            feature_types=self.feature_types_in_,
             name=name,
             selector=selector,
         )
 
     def _graph_from_tree(self, tree, feature_names=None, max_depth=None):
-        """ Adapted from:
-            https://github.com/scikit-learn/scikit-learn/blob/79bdc8f711d0af225ed6be9fdb708cea9f98a910/sklearn/tree/export.py
+        """Adapted from:
+        https://github.com/scikit-learn/scikit-learn/blob/79bdc8f711d0af225ed6be9fdb708cea9f98a910/sklearn/tree/export.py
         """
         tree_ = tree.tree_
         nodes = []
@@ -473,21 +556,21 @@ class BaseShallowDecisionTree:
 
 
 class RegressionTree(BaseShallowDecisionTree, RegressorMixin, ExplainerMixin):
-    """ Regression tree with shallow depth. """
+    """Regression tree with shallow depth."""
 
-    def __init__(self, max_depth=3, feature_names=None, feature_types=None, **kwargs):
-        """ Initializes tree with low depth.
+    def __init__(self, feature_names=None, feature_types=None, max_depth=3, **kwargs):
+        """Initializes tree with low depth.
 
         Args:
-            max_depth: Max depth of tree.
             feature_names: List of feature names.
             feature_types: List of feature types.
-            **kwargs: Kwargs sent to fit() method of tree.
+            max_depth: Max depth of tree.
+            **kwargs: Kwargs sent to __init__() method of tree.
         """
         super().__init__(
-            max_depth=max_depth,
             feature_names=feature_names,
             feature_types=feature_types,
+            max_depth=max_depth,
             **kwargs
         )
 
@@ -495,7 +578,7 @@ class RegressionTree(BaseShallowDecisionTree, RegressorMixin, ExplainerMixin):
         return self.sk_model_
 
     def fit(self, X, y):
-        """ Fits model to provided instances.
+        """Fits model to provided instances.
 
         Args:
             X: Numpy array for training instances.
@@ -509,21 +592,21 @@ class RegressionTree(BaseShallowDecisionTree, RegressorMixin, ExplainerMixin):
 
 
 class ClassificationTree(BaseShallowDecisionTree, ClassifierMixin, ExplainerMixin):
-    """ Classification tree with shallow depth. """
+    """Classification tree with shallow depth."""
 
-    def __init__(self, max_depth=3, feature_names=None, feature_types=None, **kwargs):
-        """ Initializes tree with low depth.
+    def __init__(self, feature_names=None, feature_types=None, max_depth=3, **kwargs):
+        """Initializes tree with low depth.
 
         Args:
-            max_depth: Max depth of tree.
             feature_names: List of feature names.
             feature_types: List of feature types.
-            **kwargs: Kwargs sent to fit() method of tree.
+            max_depth: Max depth of tree.
+            **kwargs: Kwargs sent to __init__() method of tree.
         """
         super().__init__(
-            max_depth=max_depth,
             feature_names=feature_names,
             feature_types=feature_types,
+            max_depth=max_depth,
             **kwargs
         )
 
@@ -531,7 +614,7 @@ class ClassificationTree(BaseShallowDecisionTree, ClassifierMixin, ExplainerMixi
         return self.sk_model_
 
     def fit(self, X, y):
-        """ Fits model to provided instances.
+        """Fits model to provided instances.
 
         Args:
             X: Numpy array for training instances.
@@ -544,7 +627,7 @@ class ClassificationTree(BaseShallowDecisionTree, ClassifierMixin, ExplainerMixi
         return super().fit(X, y)
 
     def predict_proba(self, X):
-        """ Probability estimates on provided instances.
+        """Probability estimates on provided instances.
 
         Args:
             X: Numpy array for instances.
@@ -552,5 +635,12 @@ class ClassificationTree(BaseShallowDecisionTree, ClassifierMixin, ExplainerMixi
         Returns:
             Probability estimate of instance for each class.
         """
-        X, _, _, _ = unify_data(X, None, self.feature_names, self.feature_types)
+
+        check_is_fitted(self, "has_fitted_")
+
+        X, n_samples = preclean_X(X, self.feature_names_in_, self.feature_types_in_)
+        X, _, _ = unify_data(
+            X, n_samples, self.feature_names_in_, self.feature_types_in_, False, 0
+        )
+
         return self._model().predict_proba(X)
