@@ -1,29 +1,28 @@
-# Copyright (c) 2019 Microsoft Corporation
+# Copyright (c) 2023 The InterpretML Contributors
 # Distributed under the MIT software license
 
 
 from sklearn.ensemble import RandomForestRegressor
-from sklearn.datasets import load_boston
-from ..treeinterpreter import TreeInterpreter
+from sklearn.datasets import load_diabetes
+from interpret.greybox import TreeInterpreter
 
 import pytest
 
 
-# TODO: Stop ignoring when treeinterpreter updates upstream.
-@pytest.mark.skip
 def test_that_tree_works():
     from treeinterpreter import treeinterpreter as ti
+
     # Code below compares refactored blog post to our wrapper implementation.
     # http://blog.datadive.net/random-forest-interpretation-with-scikit-learn/
 
     # Fit tree
-    boston = load_boston()
+    dataset = load_diabetes()
     rf = RandomForestRegressor()
-    X, y = boston.data[:300], boston.target[:300]
-    feature_names = boston.feature_names
+    X, y = dataset.data[:300], dataset.target[:300]
+    feature_names = dataset.feature_names
 
-    X_new = boston.data[[300, 309]]
-    y_new = boston.target[[300, 309]]
+    X_new = dataset.data[[300, 309]]
+    y_new = dataset.target[[300, 309]]
     rf.fit(X, y)
 
     # Build expected local explanation
