@@ -1,8 +1,8 @@
-# Copyright (c) 2019 Microsoft Corporation
+# Copyright (c) 2023 The InterpretML Contributors
 # Distributed under the MIT software license
 
-from ..glassbox import LogisticRegression
-from .utils import synthetic_classification
+from interpret.glassbox import LogisticRegression
+from interpret.test.utils import synthetic_classification
 
 
 # TODO: Harden these tests later to check content from data method.
