@@ -1,4 +1,4 @@
-# Copyright (c) 2019 Microsoft Corporation
+# Copyright (c) 2023 The InterpretML Contributors
 # Distributed under the MIT software license
 
 from ..data import ClassHistogram, Marginal
@@ -27,26 +27,36 @@ from pandas.core.generic import NDFrame
 from plotly import graph_objs as go
 from sklearn.base import is_classifier
 
+import sys
+
 
 def get_all_explainers():
-    data_explainer_classes = [ClassHistogram, Marginal]
-    perf_explainer_classes = [ROC, PR, RegressionPerf]
+    # True means run on classification.  False means run on regression
+    data_explainer_classes = [(ClassHistogram, None), (Marginal, None)]
+    perf_explainer_classes = [(ROC, True), (PR, True), (RegressionPerf, False)]
     model_explainer_classes = [
-        ClassificationTree,
-        DecisionListClassifier,
-        LogisticRegression,
-        ExplainableBoostingClassifier,
-        RegressionTree,
-        LinearRegression,
-        ExplainableBoostingRegressor,
+        (ClassificationTree, True),
+        (RegressionTree, False),
+        (LogisticRegression, True),
+        (LinearRegression, False),
+        (ExplainableBoostingClassifier, True),
+        (ExplainableBoostingRegressor, False),
+    ]
+    specific_explainer_classes = [
+        (TreeInterpreter, True),
+        (TreeInterpreter, False),
+        (ShapTree, True),
+        (ShapTree, False),
     ]
-    # specific_explainer_classes = [TreeInterpreter, ShapTree]
-    specific_explainer_classes = [ShapTree]
     blackbox_explainer_classes = [
-        LimeTabular,
-        ShapKernel,
-        MorrisSensitivity,
-        PartialDependence,
+        (LimeTabular, True),
+        (LimeTabular, False),
+        (ShapKernel, True),
+        (ShapKernel, False),
+        (MorrisSensitivity, True),
+        (MorrisSensitivity, False),
+        (PartialDependence, True),
+        (PartialDependence, False),
         # PermutationImportance
     ]
     all_explainers = []
@@ -56,6 +66,12 @@ def get_all_explainers():
     all_explainers.extend(data_explainer_classes)
     all_explainers.extend(perf_explainer_classes)
 
+    if sys.version_info[0] <= 3 and sys.version_info[1] < 10:
+        # skope-rules doesn't work in python 3.10 as of March 2023,
+        # although there was a PR accepted to change this a few weeks ago
+        # so the version after 1.0.1 should work
+        all_explainers.append((DecisionListClassifier, True))
+
     return all_explainers
 
 
@@ -96,10 +112,6 @@ def _synthetic(mode="regression"):
     return dataset
 
 
-def boston_regression():
-    return None
-
-
 def iris_classification():
     from sklearn.datasets import load_iris
 
