@@ -1,4 +1,4 @@
-from ..all import gen_perf_dicts
+from interpret.utils import gen_perf_dicts
 
 import numpy as np
 
@@ -20,7 +20,12 @@ def test_gen_perf_dicts_regression():
 
 def test_gen_perf_dicts_classification():
     y = np.array([0, 2])
-    scores = np.array([[0.9, 0.06, 0.04], [0.1, 0.5, 0.4],])
+    scores = np.array(
+        [
+            [0.9, 0.06, 0.04],
+            [0.1, 0.5, 0.4],
+        ]
+    )
     expected_predicted = np.array([0, 1])
     expected_actual_score = np.array([0.9, 0.4])
     expected_predicted_score = np.array([0.9, 0.5])
