@@ -1,10 +1,10 @@
-# Copyright (c) 2019 Microsoft Corporation
+# Copyright (c) 2023 The InterpretML Contributors
 # Distributed under the MIT software license
 
 # NOTE: Render call is tested with other providers elsewhere.
 
 import plotly.graph_objects as go
-from ..inline import _build_viz_figure
+from interpret.visual.inline import _build_viz_figure
 import pytest
 
 
