@@ -1,4 +1,4 @@
-// Copyright (c) 2018 Microsoft Corporation
+// Copyright (c) 2023 The InterpretML Contributors
 // Licensed under the MIT license.
 // Author: Paul Koch <code@koch.ninja>
 
@@ -7,7 +7,7 @@
 
 #include <assert.h>
 
-#include "ebm_native.h" // LOG_MESSAGE_FUNCTION
+#include "ebm_native.h" // TraceEbm
 
 #ifdef __cplusplus
 extern "C" {
@@ -27,37 +27,37 @@ extern "C" {
 
 #endif // __clang__
 
-extern const char g_trueString[];
-extern const char g_falseString[];
+extern const char g_sTrue[];
+extern const char g_sFalse[];
 
-inline static const char * ObtainTruth(const BoolEbmType b) {
-   return EBM_FALSE != b ? g_trueString : g_falseString;
+inline static const char * ObtainTruth(const BoolEbm b) {
+   return EBM_FALSE != b ? g_sTrue : g_sFalse;
 }
 
-extern TraceEbmType g_traceLevel;
+extern TraceEbm g_traceLevel;
 
-extern void InteralLogWithArguments(const TraceEbmType traceLevel, const char * const pOriginalMessage, ...);
-extern void InteralLogWithoutArguments(const TraceEbmType traceLevel, const char * const pOriginalMessage);
+extern void InteralLogWithArguments(const TraceEbm traceLevel, const char * const sMessage, ...);
+extern void InteralLogWithoutArguments(const TraceEbm traceLevel, const char * const sMessage);
 extern void LogAssertFailure(
    const unsigned long long lineNumber,
-   const char * const fileName,
-   const char * const functionName,
-   const char * const assertText
+   const char * const sFileName,
+   const char * const sFunctionName,
+   const char * const sAssertText
 ) LOGGING_ANALYZER_NORETURN ;
 
 // We use separate macros for LOG_0 (zero parameters) and LOG_N (variadic parameters) because having zero parameters is non-standardized in C++11
 // In C++20, there will be __VA_OPT__, but I don't want to take a dependency on such a new standard yet
 // In GCC, you can use ##__VA_ARGS__, but it's non-standard
-// In C++11, you can use (but it's more complicated and I can't generate "const static char" string arrays by default):
-// constexpr size_t LOG__cArguments = std::tuple_size<decltype(std::make_tuple(__VA_ARGS__))>::value; // this gets the number of arguments
-// static const char * LOG__originalMessage = std::get<0>(std::make_tuple(__VA_ARGS__)); // this gets a specific argument by index.  In C++14 it should 
-// have a constexpr version
+// In C++11, you can use (but it's more complicated and I can't generate "static const char" string arrays by default):
+// static constexpr size_t LOG__cArguments = std::tuple_size<decltype(std::make_tuple(__VA_ARGS__))>::value; // this gets the number of arguments
+// static const char * LOG__sMessage = std::get<0>(std::make_tuple(__VA_ARGS__)); // this gets a specific argument by index.  In C++14 it should 
+// have a static constexpr version
 
 // use a MACRO for LOG_0/LOG_N(..) and LOG_COUNTED_0/LOG_COUNTED_N(..) instead of an inline because:
 //   1) we can use static_assert on the log level
 //   2) we can get the number of variadic arguments at compile time, allowing the call to InteralLogWithArguments to be optimized away in cases where we 
 //      have a simple string
-//   3) we can put our input string into a const static char LOG__originalMessage[] instead of keeping it as a string, which is useful in some languages 
+//   3) we can put our input string into a static const char LOG__sMessage[] instead of keeping it as a string, which is useful in some languages 
 //      because I think strings are not const, so by default get put into the read/write data segment instead of readonly
 //   3) our variadic arguments won't be evaluated unless they are necessary (log level is set high enough).  If we had inlined them, they might have 
 //      needed to be evaluated, depending on the inputs
@@ -72,47 +72,47 @@ extern void LogAssertFailure(
 // we only use each input parameter once, which avoids pre and post decrement issues with macros
 //
 // we use LOG__ prefixes for any variables that we define to avoid collisions with any code we get inserted into
-#define LOG_0(traceLevel, pLogMessage) \
+#define LOG_0(traceLevel, sMessage) \
    do { \
-      const TraceEbmType LOG__traceLevel = (traceLevel); \
-      static_assert(TraceLevelOff < LOG__traceLevel, "traceLevel can't be TraceLevelOff or lower for call to LOG_0(traceLevel, pLogMessage, ...)"); \
-      static_assert(LOG__traceLevel <= TraceLevelVerbose, "traceLevel can't be higher than TraceLevelVerbose for call to LOG_0(traceLevel, pLogMessage, ...)"); \
+      const TraceEbm LOG__traceLevel = (traceLevel); \
+      static_assert(Trace_Off < LOG__traceLevel, "traceLevel can't be Trace_Off or lower for call to LOG_0(traceLevel, sMessage, ...)"); \
+      static_assert(LOG__traceLevel <= Trace_Verbose, "traceLevel can't be higher than Trace_Verbose for call to LOG_0(traceLevel, sMessage, ...)"); \
       if(LOG__traceLevel <= g_traceLevel) { \
-         const static char LOG__originalMessage[] = pLogMessage; \
-         InteralLogWithoutArguments(LOG__traceLevel, LOG__originalMessage); \
+         static const char LOG__sMessage[] = sMessage; \
+         InteralLogWithoutArguments(LOG__traceLevel, LOG__sMessage); \
       } \
    } while( (void)0, 0)
 
-#define LOG_N(traceLevel, pLogMessage, ...) \
+#define LOG_N(traceLevel, sMessage, ...) \
    do { \
-      const TraceEbmType LOG__traceLevel = (traceLevel); \
-      static_assert(TraceLevelOff < LOG__traceLevel, "traceLevel can't be TraceLevelOff or lower for call to LOG_N(traceLevel, pLogMessage, ...)"); \
-      static_assert(LOG__traceLevel <= TraceLevelVerbose, \
-         "traceLevel can't be higher than TraceLevelVerbose for call to LOG_N(traceLevel, pLogMessage, ...)"); \
+      const TraceEbm LOG__traceLevel = (traceLevel); \
+      static_assert(Trace_Off < LOG__traceLevel, "traceLevel can't be Trace_Off or lower for call to LOG_N(traceLevel, sMessage, ...)"); \
+      static_assert(LOG__traceLevel <= Trace_Verbose, \
+         "traceLevel can't be higher than Trace_Verbose for call to LOG_N(traceLevel, sMessage, ...)"); \
       if(LOG__traceLevel <= g_traceLevel) { \
-         const static char LOG__originalMessage[] = pLogMessage; \
-         InteralLogWithArguments(LOG__traceLevel, LOG__originalMessage, __VA_ARGS__); \
+         static const char LOG__sMessage[] = sMessage; \
+         InteralLogWithArguments(LOG__traceLevel, LOG__sMessage, __VA_ARGS__); \
       } \
    } while( (void)0, 0)
 
-#define LOG_COUNTED_0(pLogCountDecrement, traceLevelBefore, traceLevelAfter, pLogMessage) \
+#define LOG_COUNTED_0(pLogCountDecrement, traceLevelBefore, traceLevelAfter, sMessage) \
    do { \
-      const TraceEbmType LOG__traceLevelBefore = (traceLevelBefore); \
-      static_assert(TraceLevelOff < LOG__traceLevelBefore, \
-         "traceLevelBefore can't be TraceLevelOff or lower for call to LOG_COUNTED_0(pLogCount, traceLevelBefore, traceLevelAfter, pLogMessage, ...)"); \
-      static_assert(LOG__traceLevelBefore <= TraceLevelVerbose, \
-         "traceLevelBefore can't be higher than TraceLevelVerbose for call to LOG_COUNTED_0(pLogCount, traceLevelBefore, traceLevelAfter, pLogMessage, ...)"); \
-      const TraceEbmType LOG__traceLevelAfter = (traceLevelAfter); \
-      static_assert(TraceLevelOff < LOG__traceLevelAfter, \
-         "traceLevelAfter can't be TraceLevelOff or lower for call to LOG_COUNTED_0(pLogCount, traceLevelBefore, traceLevelAfter, pLogMessage, ...)"); \
-      static_assert(LOG__traceLevelAfter <= TraceLevelVerbose, \
-         "traceLevelAfter can't be higher than TraceLevelVerbose for call to LOG_COUNTED_0(pLogCount, traceLevelBefore, traceLevelAfter, pLogMessage, ...)"); \
+      const TraceEbm LOG__traceLevelBefore = (traceLevelBefore); \
+      static_assert(Trace_Off < LOG__traceLevelBefore, \
+         "traceLevelBefore can't be Trace_Off or lower for call to LOG_COUNTED_0(pLogCount, traceLevelBefore, traceLevelAfter, sMessage, ...)"); \
+      static_assert(LOG__traceLevelBefore <= Trace_Verbose, \
+         "traceLevelBefore can't be higher than Trace_Verbose for call to LOG_COUNTED_0(pLogCount, traceLevelBefore, traceLevelAfter, sMessage, ...)"); \
+      const TraceEbm LOG__traceLevelAfter = (traceLevelAfter); \
+      static_assert(Trace_Off < LOG__traceLevelAfter, \
+         "traceLevelAfter can't be Trace_Off or lower for call to LOG_COUNTED_0(pLogCount, traceLevelBefore, traceLevelAfter, sMessage, ...)"); \
+      static_assert(LOG__traceLevelAfter <= Trace_Verbose, \
+         "traceLevelAfter can't be higher than Trace_Verbose for call to LOG_COUNTED_0(pLogCount, traceLevelBefore, traceLevelAfter, sMessage, ...)"); \
       static_assert(LOG__traceLevelBefore < LOG__traceLevelAfter, \
          "We only support increasing the required trace level after N iterations. It doesn't make sense to have equal values, otherwise just use LOG_0(..)"); \
-      const TraceEbmType LOG__traceLevel = g_traceLevel; \
+      const TraceEbm LOG__traceLevel = g_traceLevel; \
       if(LOG__traceLevelBefore <= LOG__traceLevel) { \
          do { \
-            TraceEbmType LOG__traceLevelLogging; \
+            TraceEbm LOG__traceLevelLogging; \
             if(LOG__traceLevel < LOG__traceLevelAfter) { \
                int * const LOG__pLogCountDecrement = (pLogCountDecrement); \
                const int LOG__logCount = *LOG__pLogCountDecrement - 1; \
@@ -124,30 +124,30 @@ extern void LogAssertFailure(
             } else { \
                LOG__traceLevelLogging = LOG__traceLevelAfter; \
             }\
-            const static char LOG__originalMessage[] = pLogMessage; \
-            InteralLogWithoutArguments(LOG__traceLevelLogging, LOG__originalMessage); \
+            static const char LOG__sMessage[] = sMessage; \
+            InteralLogWithoutArguments(LOG__traceLevelLogging, LOG__sMessage); \
          } while( (void)0, 0); \
       } \
    } while( (void)0, 0)
 
-#define LOG_COUNTED_N(pLogCountDecrement, traceLevelBefore, traceLevelAfter, pLogMessage, ...) \
+#define LOG_COUNTED_N(pLogCountDecrement, traceLevelBefore, traceLevelAfter, sMessage, ...) \
    do { \
-      const TraceEbmType LOG__traceLevelBefore = (traceLevelBefore); \
-      static_assert(TraceLevelOff < LOG__traceLevelBefore, \
-         "traceLevelBefore can't be TraceLevelOff or lower for call to LOG_COUNTED_N(pLogCount, traceLevelBefore, traceLevelAfter, pLogMessage, ...)"); \
-      static_assert(LOG__traceLevelBefore <= TraceLevelVerbose, \
-         "traceLevelBefore can't be higher than TraceLevelVerbose for call to LOG_COUNTED_N(pLogCount, traceLevelBefore, traceLevelAfter, pLogMessage, ...)"); \
-      const TraceEbmType LOG__traceLevelAfter = (traceLevelAfter); \
-      static_assert(TraceLevelOff < LOG__traceLevelAfter, \
-         "traceLevelAfter can't be TraceLevelOff or lower for call to LOG_COUNTED_N(pLogCount, traceLevelBefore, traceLevelAfter, pLogMessage, ...)"); \
-      static_assert(LOG__traceLevelAfter <= TraceLevelVerbose, \
-         "traceLevelAfter can't be higher than TraceLevelVerbose for call to LOG_COUNTED_N(pLogCount, traceLevelBefore, traceLevelAfter, pLogMessage, ...)"); \
+      const TraceEbm LOG__traceLevelBefore = (traceLevelBefore); \
+      static_assert(Trace_Off < LOG__traceLevelBefore, \
+         "traceLevelBefore can't be Trace_Off or lower for call to LOG_COUNTED_N(pLogCount, traceLevelBefore, traceLevelAfter, sMessage, ...)"); \
+      static_assert(LOG__traceLevelBefore <= Trace_Verbose, \
+         "traceLevelBefore can't be higher than Trace_Verbose for call to LOG_COUNTED_N(pLogCount, traceLevelBefore, traceLevelAfter, sMessage, ...)"); \
+      const TraceEbm LOG__traceLevelAfter = (traceLevelAfter); \
+      static_assert(Trace_Off < LOG__traceLevelAfter, \
+         "traceLevelAfter can't be Trace_Off or lower for call to LOG_COUNTED_N(pLogCount, traceLevelBefore, traceLevelAfter, sMessage, ...)"); \
+      static_assert(LOG__traceLevelAfter <= Trace_Verbose, \
+         "traceLevelAfter can't be higher than Trace_Verbose for call to LOG_COUNTED_N(pLogCount, traceLevelBefore, traceLevelAfter, sMessage, ...)"); \
       static_assert(LOG__traceLevelBefore < LOG__traceLevelAfter, \
          "We only support increasing the required trace level after N iterations and it doesn't make sense to have equal values, otherwise just use LOG_N(...)"); \
-      const TraceEbmType LOG__traceLevel = g_traceLevel; \
+      const TraceEbm LOG__traceLevel = g_traceLevel; \
       if(LOG__traceLevelBefore <= LOG__traceLevel) { \
          do { \
-            TraceEbmType LOG__traceLevelLogging; \
+            TraceEbm LOG__traceLevelLogging; \
             if(LOG__traceLevel < LOG__traceLevelAfter) { \
                int * const LOG__pLogCountDecrement = (pLogCountDecrement); \
                const int LOG__logCount = *LOG__pLogCountDecrement - 1; \
@@ -159,8 +159,8 @@ extern void LogAssertFailure(
             } else { \
                LOG__traceLevelLogging = LOG__traceLevelAfter; \
             }\
-            const static char LOG__originalMessage[] = pLogMessage; \
-            InteralLogWithArguments(LOG__traceLevelLogging, LOG__originalMessage, __VA_ARGS__); \
+            static const char LOG__sMessage[] = sMessage; \
+            InteralLogWithArguments(LOG__traceLevelLogging, LOG__sMessage, __VA_ARGS__); \
          } while( (void)0, 0); \
       } \
    } while( (void)0, 0)
