@@ -1,11 +1,11 @@
-// Copyright (c) 2018 Microsoft Corporation
+// Copyright (c) 2023 The InterpretML Contributors
 // Licensed under the MIT license.
 // Author: Paul Koch <code@koch.ninja>
 
 #define _CRT_SECURE_NO_DEPRECATE
 
 #include <stddef.h> // size_t, ptrdiff_t
-#include <inttypes.h> // any of the fixed size types like int64_t
+#include <inttypes.h> // fixed sized integer types and printf strings.  Includes stdint.h
 #include <limits> // numeric_limits
 #include <type_traits> // std::is_standard_layout, std::is_integral
 #include <stdlib.h> // malloc, free
